@@ -1,21 +1,44 @@
-/* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
+/* Copyright (c) 2001 Matej Pfajfar.
+ * Copyright (c) 2001-2004, Roger Dingledine.
+ * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: aes.c,v 1.18 2005/05/07 05:55:05 nickm Exp $ */
-const char aes_c_id[] = "$Id: aes.c,v 1.18 2005/05/07 05:55:05 nickm Exp $";
+/* $Id: aes.c,v 1.32 2006/02/09 05:46:48 arma Exp $ */
+const char aes_c_id[] = "$Id: aes.c,v 1.32 2006/02/09 05:46:48 arma Exp $";
 
 /**
  * \file aes.c
- *
- * \brief Implementation of a simple AES counter mode.
+ * \brief Implements the AES cipher (with 128-bit keys and blocks),
+ * and a counter-mode stream cipher on top of AES.  This code is
+ * taken from the main Rijndael distribution.  (We include this
+ * because many people are running older versions of OpenSSL without
+ * AES support.)
  **/
 
 #include "orconfig.h"
+#include <openssl/opensslv.h>
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 #include "aes.h"
 #include "util.h"
+#include "log.h"
+
+/* Use OpenSSL's AES if we're running 0.9.7 or later.  (The f at the end of
+ * the version below means "release"; see opensslv.h) */
+#if OPENSSL_VERSION_NUMBER >= 0x0090700fl
+#define USE_OPENSSL_AES
+#include <openssl/aes.h>
+#include <openssl/evp.h>
+#endif
+
+/* Benchmarking suggests that using the built-in rijndael below is
+ * significantly faster than using OpenSSL's EVP code (by about 27%)
+ * and faster than using OpenSSL's AES functions (by about 19%).
+ * The counter-mode optimization saves around 5%.
+ */
+#undef USE_OPENSSL_AES
+#undef USE_OPENSSL_EVP
+#define USE_RIJNDAEL_COUNTER_OPTIMIZATION
 
 /*======================================================================*/
 /* From rijndael-alg-fst.h */
@@ -24,19 +47,34 @@ typedef uint64_t u64;
 typedef uint32_t u32;
 typedef uint8_t u8;
 
+#ifndef USE_OPENSSL_AES
 #define MAXKC   (256/32)
 #define MAXKB   (256/8)
 #define MAXNR   14
 
-static int rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBits);
-static void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 ct[16]);
+static int rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/],
+                               const u8 cipherKey[], int keyBits);
+#ifdef USE_RIJNDAEL_COUNTER_OPTIMIZATION
+static void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr,
+                            u32 ctr1, u32 ctr0, u8 ct[16]);
+#else
+static void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr,
+                            const u8 pt[16], u8 ct[16]);
+#endif
+#endif
 
 /*======================================================================*/
 /* Interface to AES code, and counter implementation */
 
 struct aes_cnt_cipher {
+#if defined(USE_OPENSSL_EVP)
+  EVP_CIPHER_CTX key;
+#elif defined(USE_OPENSSL_AES)
+  AES_KEY key;
+#else
   u32 rk[4*(MAXNR+1)];
   int nr;
+#endif
   u32 counter1;
   u32 counter0;
   u8 buf[16];
@@ -47,9 +85,20 @@ struct aes_cnt_cipher {
  * Helper function: set <b>cipher</b>'s internal buffer to the encrypted
  * value of the current counter.
  */
-static void
+static INLINE void
 _aes_fill_buf(aes_cnt_cipher_t *cipher)
 {
+  /* We don't currently use OpenSSL's counter mode implementation because:
+   *  1) some versions have known bugs
+   *  2) its attitude towards IVs is not our own
+   *  3) changing the counter position was not trivial, last time I looked.
+   * None of these issues are insurmountable in principle.
+   */
+#if (!defined(USE_OPENSSL_EVP) && !defined(USE_OPENSSL_AES) && \
+     defined(USE_RIJNDAEL_COUNTER_OPTIMIZATION))
+  rijndaelEncrypt(cipher->rk, cipher->nr,
+                  cipher->counter1, cipher->counter0, cipher->buf);
+#else
   u32 counter0 = cipher->counter0;
   u32 counter1 = cipher->counter1;
   u8 buf[16];
@@ -63,18 +112,26 @@ _aes_fill_buf(aes_cnt_cipher_t *cipher)
   buf[ 9] = (counter1 >> 16) & 0xff;
   buf[ 8] = (counter1 >> 24) & 0xff;
 
+#if defined(USE_OPENSSL_EVP)
+  {
+    int outl=16, inl=16;
+    EVP_EncryptUpdate(&cipher->key, cipher->buf, &outl, buf, inl);
+  }
+#elif defined(USE_OPENSSL_AES)
+  AES_encrypt(buf, cipher->buf, &cipher->key);
+#else
   rijndaelEncrypt(cipher->rk, cipher->nr, buf, cipher->buf);
+#endif
+#endif
 }
 
 /**
  * Return a newly allocated counter-mode AES128 cipher implementation.
  */
 aes_cnt_cipher_t*
-aes_new_cipher()
+aes_new_cipher(void)
 {
-  aes_cnt_cipher_t* result = tor_malloc(sizeof(aes_cnt_cipher_t));
-  memset(result->rk, 0, 4*(MAXNR+1));
-  memset(result->buf, 0, 16);
+  aes_cnt_cipher_t* result = tor_malloc_zero(sizeof(aes_cnt_cipher_t));
 
   return result;
 }
@@ -86,8 +143,21 @@ aes_new_cipher()
 void
 aes_set_key(aes_cnt_cipher_t *cipher, const char *key, int key_bits)
 {
+#if defined(USE_OPENSSL_EVP)
+  const EVP_CIPHER *c;
+  switch (key_bits) {
+    case 128: c = EVP_aes_128_ecb(); break;
+    case 192: c = EVP_aes_192_ecb(); break;
+    case 256: c = EVP_aes_256_ecb(); break;
+    default: tor_assert(0);
+  }
+  EVP_EncryptInit(&cipher->key, c, (const unsigned char*)key, NULL);
+#elif defined(USE_OPENSSL_AES)
+  AES_set_encrypt_key((const unsigned char *)key, key_bits, &(cipher->key));
+#else
   cipher->nr = rijndaelKeySetupEnc(cipher->rk, (const unsigned char*)key,
                                    key_bits);
+#endif
   cipher->counter0 = 0;
   cipher->counter1 = 0;
   cipher->pos = 0;
@@ -100,8 +170,11 @@ void
 aes_free_cipher(aes_cnt_cipher_t *cipher)
 {
   assert(cipher);
+#ifdef USE_OPENSSL_EVP
+  EVP_CIPHER_CTX_cleanup(&cipher->key);
+#endif
   memset(cipher, 0, sizeof(cipher));
-  free(cipher);
+  tor_free(cipher);
 }
 
 /** Encrypt <b>len</b> bytes from <b>input</b>, storing the result in
@@ -109,7 +182,8 @@ aes_free_cipher(aes_cnt_cipher_t *cipher)
  * by <b>len</b> bytes as it encrypts.
  */
 void
-aes_crypt(aes_cnt_cipher_t *cipher, const char *input, size_t len, char *output)
+aes_crypt(aes_cnt_cipher_t *cipher, const char *input, size_t len,
+          char *output)
 {
   int c = cipher->pos;
   if (!len) return;
@@ -155,6 +229,7 @@ aes_adjust_counter(aes_cnt_cipher_t *cipher, long delta)
   aes_set_counter(cipher, counter);
 }
 
+#ifndef USE_OPENSSL_AES
 /*======================================================================*/
 /* From rijndael-alg-fst.c */
 
@@ -551,7 +626,9 @@ static const u32 rcon[] = {
  *
  * @return      the number of rounds for the given cipher key size.
  */
-int rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBits) {
+int
+rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBits)
+{
         int i = 0;
         u32 temp;
 
@@ -632,7 +709,14 @@ int rijndaelKeySetupEnc(u32 rk[/*4*(Nr + 1)*/], const u8 cipherKey[], int keyBit
         return 0;
 }
 
-void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 ct[16]) {
+#ifdef USE_RIJNDAEL_COUNTER_OPTIMIZATION
+void
+rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, u32 ctr1, u32 ctr0, u8 ct[16])
+#else
+void
+rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 ct[16])
+#endif
+{
         u32 s0, s1, s2, s3, t0, t1, t2, t3;
 #ifndef FULL_UNROLL
     int r;
@@ -642,10 +726,18 @@ void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 c
          * map byte array block to cipher state
          * and add initial round key:
          */
+#ifdef USE_RIJNDAEL_COUNTER_OPTIMIZATION
+        s0 = rk[0];
+        s1 = rk[1];
+        s2 = ctr1 ^ rk[2];
+        s3 = ctr0 ^ rk[3];
+#else
         s0 = GETU32(pt     ) ^ rk[0];
         s1 = GETU32(pt +  4) ^ rk[1];
         s2 = GETU32(pt +  8) ^ rk[2];
         s3 = GETU32(pt + 12) ^ rk[3];
+#endif
+
 #ifdef FULL_UNROLL
     /* round 1: */
         t0 = Te0[s0 >> 24] ^ Te1[(s1 >> 16) & 0xff] ^ Te2[(s2 >>  8) & 0xff] ^ Te3[s3 & 0xff] ^ rk[ 4];
@@ -812,4 +904,5 @@ void rijndaelEncrypt(const u32 rk[/*4*(Nr + 1)*/], int Nr, const u8 pt[16], u8 c
                 rk[3];
         PUTU32(ct + 12, s3);
 }
+#endif
 
