@@ -1,19 +1,22 @@
 /* Copyright 2003 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
+ * Copyright 2004-2006 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: tortls.c,v 1.94 2005/05/07 05:55:05 nickm Exp $ */
-const char tortls_c_id[] = "$Id: tortls.c,v 1.94 2005/05/07 05:55:05 nickm Exp $";
+/* $Id: tortls.c,v 1.118 2006/03/05 09:50:25 nickm Exp $ */
+const char tortls_c_id[] =
+  "$Id: tortls.c,v 1.118 2006/03/05 09:50:25 nickm Exp $";
 
 /**
  * \file tortls.c
- *
- * \brief TLS wrappers for Tor.
+ * \brief Wrapper functions to present a consistent interface to
+ * TLS, SSL, and X.509 functions from OpenSSL.
  **/
+
 /* (Unlike other tor functions, these
  * are prefixed with tor_ in order to avoid conflicting with OpenSSL
  * functions and variables.)
  */
 
+#include "orconfig.h"
 #include "./crypto.h"
 #include "./tortls.h"
 #include "./util.h"
@@ -21,7 +24,8 @@ const char tortls_c_id[] = "$Id: tortls.c,v 1.94 2005/05/07 05:55:05 nickm Exp $
 #include <string.h>
 
 /* Copied from or.h */
-#define LEGAL_NICKNAME_CHARACTERS "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
+#define LEGAL_NICKNAME_CHARACTERS \
+  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
 
 #include <assert.h>
 #include <openssl/ssl.h>
@@ -33,15 +37,16 @@ const char tortls_c_id[] = "$Id: tortls.c,v 1.94 2005/05/07 05:55:05 nickm Exp $
 /** How long do identity certificates live? (sec) */
 #define IDENTITY_CERT_LIFETIME  (365*24*60*60)
 
-typedef struct tor_tls_context_st {
+/* DOCDOC */
+typedef struct tor_tls_context_t {
   SSL_CTX *ctx;
   SSL_CTX *client_only_ctx;
-} tor_tls_context;
+} tor_tls_context_t;
 
 /** Holds a SSL object and its associated data.  Members are only
  * accessed from within tortls.c.
  */
-struct tor_tls_st {
+struct tor_tls_t {
   SSL *ssl; /**< An OpenSSL SSL object. */
   int socket; /**< The underlying file descriptor for this TLS connection. */
   enum {
@@ -50,7 +55,8 @@ struct tor_tls_st {
   } state; /**< The current SSL state, depending on which operations have
             * completed successfully. */
   int isServer;
-  size_t wantwrite_n; /**< 0 normally, >0 if we returned wantwrite last time. */
+  size_t wantwrite_n; /**< 0 normally, >0 if we returned wantwrite last
+                       * time. */
 };
 
 static X509* tor_tls_create_certificate(crypto_pk_env_t *rsa,
@@ -61,7 +67,7 @@ static X509* tor_tls_create_certificate(crypto_pk_env_t *rsa,
 
 /** Global tls context. We keep it here because nobody else needs to
  * touch it. */
-static tor_tls_context *global_tls_context = NULL;
+static tor_tls_context_t *global_tls_context = NULL;
 /** True iff tor_tls_init() has been called. */
 static int tls_library_is_initialized = 0;
 
@@ -88,9 +94,10 @@ tls_log_errors(int severity, const char *doing)
     func = (const char*)ERR_func_error_string(err);
     if (!msg) msg = "(null)";
     if (doing) {
-      log(severity, "TLS error while %s: %s (in %s:%s)", doing, msg, lib,func);
+      log(severity, LD_NET, "TLS error while %s: %s (in %s:%s)",
+          doing, msg, lib,func);
     } else {
-      log(severity, "TLS error: %s (in %s:%s)", msg, lib, func);
+      log(severity, LD_NET, "TLS error: %s (in %s:%s)", msg, lib, func);
     }
   }
 }
@@ -109,7 +116,7 @@ tls_log_errors(int severity, const char *doing)
  * current action as <b>doing</b>.
  */
 static int
-tor_tls_get_error(tor_tls *tls, int r, int extra,
+tor_tls_get_error(tor_tls_t *tls, int r, int extra,
                   const char *doing, int severity)
 {
   int err = SSL_get_error(tls->ssl, r);
@@ -124,10 +131,11 @@ tor_tls_get_error(tor_tls *tls, int r, int extra,
       if (extra&CATCH_SYSCALL)
         return _TOR_TLS_SYSCALL;
       if (r == 0)
-        log(severity, "TLS error: unexpected close while %s", doing);
+        log(severity, LD_NET, "TLS error: unexpected close while %s", doing);
       else {
         int e = tor_socket_errno(tls->socket);
-        log(severity, "TLS error: <syscall error while %s> (errno=%d: %s)",
+        log(severity, LD_NET,
+            "TLS error: <syscall error while %s> (errno=%d: %s)",
             doing, e, tor_socket_strerror(e));
       }
       tls_log_errors(severity, doing);
@@ -135,7 +143,7 @@ tor_tls_get_error(tor_tls *tls, int r, int extra,
     case SSL_ERROR_ZERO_RETURN:
       if (extra&CATCH_ZERO)
         return _TOR_TLS_ZERORETURN;
-      log(severity, "TLS error: Zero return");
+      log(severity, LD_NET, "TLS error: Zero return");
       tls_log_errors(severity, doing);
       return TOR_TLS_ERROR;
     default:
@@ -147,12 +155,12 @@ tor_tls_get_error(tor_tls *tls, int r, int extra,
 /** Initialize OpenSSL, unless it has already been initialized.
  */
 static void
-tor_tls_init(void) {
+tor_tls_init(void)
+{
   if (!tls_library_is_initialized) {
     SSL_library_init();
     SSL_load_error_strings();
-    crypto_global_init();
-    OpenSSL_add_all_algorithms();
+    crypto_global_init(-1);
     tls_library_is_initialized = 1;
   }
 }
@@ -172,9 +180,13 @@ tor_tls_free_all(void)
  * it: We always accept peer certs and complete the handshake.  We
  * don't validate them until later.
  */
-static int always_accept_verify_cb(int preverify_ok,
-                                   X509_STORE_CTX *x509_ctx)
+static int
+always_accept_verify_cb(int preverify_ok,
+                        X509_STORE_CTX *x509_ctx)
 {
+  /* avoid "unused parameter" warning. */
+  preverify_ok = 0;
+  x509_ctx = NULL;
   return 1;
 }
 
@@ -219,23 +231,29 @@ tor_tls_create_certificate(crypto_pk_env_t *rsa,
 
   if (!(name = X509_NAME_new()))
     goto error;
-  if ((nid = OBJ_txt2nid("organizationName")) == NID_undef) goto error;
+  if ((nid = OBJ_txt2nid("organizationName")) == NID_undef)
+    goto error;
   if (!(X509_NAME_add_entry_by_NID(name, nid, MBSTRING_ASC,
-                                   (unsigned char*)"TOR", -1, -1, 0))) goto error;
+                                   (unsigned char*)"TOR", -1, -1, 0)))
+    goto error;
   if ((nid = OBJ_txt2nid("commonName")) == NID_undef) goto error;
   if (!(X509_NAME_add_entry_by_NID(name, nid, MBSTRING_ASC,
-                                   (unsigned char*)cname, -1, -1, 0))) goto error;
+                                   (unsigned char*)cname, -1, -1, 0)))
+    goto error;
   if (!(X509_set_subject_name(x509, name)))
     goto error;
 
   if (!(name_issuer = X509_NAME_new()))
     goto error;
-  if ((nid = OBJ_txt2nid("organizationName")) == NID_undef) goto error;
+  if ((nid = OBJ_txt2nid("organizationName")) == NID_undef)
+    goto error;
   if (!(X509_NAME_add_entry_by_NID(name_issuer, nid, MBSTRING_ASC,
-                                   (unsigned char*)"TOR", -1, -1, 0))) goto error;
+                                   (unsigned char*)"TOR", -1, -1, 0)))
+    goto error;
   if ((nid = OBJ_txt2nid("commonName")) == NID_undef) goto error;
   if (!(X509_NAME_add_entry_by_NID(name_issuer, nid, MBSTRING_ASC,
-                              (unsigned char*)cname_sign, -1, -1, 0))) goto error;
+                              (unsigned char*)cname_sign, -1, -1, 0)))
+    goto error;
   if (!(X509_set_issuer_name(x509, name_issuer)))
     goto error;
 
@@ -302,7 +320,7 @@ tor_tls_context_new(crypto_pk_env_t *identity,
   crypto_pk_env_t *rsa = NULL;
   crypto_dh_env_t *dh = NULL;
   EVP_PKEY *pkey = NULL;
-  tor_tls_context *result = NULL;
+  tor_tls_context_t *result = NULL;
   X509 *cert = NULL, *idcert = NULL;
   char nn2[128];
   int client_only;
@@ -326,12 +344,12 @@ tor_tls_context_new(crypto_pk_env_t *identity,
     idcert = tor_tls_create_certificate(identity, identity, nn2, nn2,
                                         IDENTITY_CERT_LIFETIME);
     if (!cert || !idcert) {
-      log(LOG_WARN, "Error creating certificate");
+      log(LOG_WARN, LD_CRYPTO, "Error creating certificate");
       goto error;
     }
   }
 
-  result = tor_malloc(sizeof(tor_tls_context));
+  result = tor_malloc(sizeof(tor_tls_context_t));
   result->ctx = result->client_only_ctx = NULL;
   for (client_only=0; client_only <= 1; ++client_only) {
     ctx = client_only ? &result->client_only_ctx : &result->ctx;
@@ -344,6 +362,9 @@ tor_tls_context_new(crypto_pk_env_t *identity,
     if (!(*ctx = SSL_CTX_new(SSLv23_method())))
       goto error;
     SSL_CTX_set_options(*ctx, SSL_OP_NO_SSLv2);
+#endif
+#ifndef ENABLE_0119_PARANOIA_A
+    SSL_CTX_set_options(*ctx, SSL_OP_SINGLE_DH_USE);
 #endif
     if (!SSL_CTX_set_cipher_list(*ctx, CIPHER_LIST))
       goto error;
@@ -402,7 +423,7 @@ tor_tls_context_new(crypto_pk_env_t *identity,
   if (result && result->client_only_ctx)
     SSL_CTX_free(result->client_only_ctx);
   if (result)
-    free(result);
+    tor_free(result);
   if (cert)
     X509_free(cert);
   if (idcert)
@@ -413,10 +434,10 @@ tor_tls_context_new(crypto_pk_env_t *identity,
 /** Create a new TLS object from a file descriptor, and a flag to
  * determine whether it is functioning as a server.
  */
-tor_tls *
+tor_tls_t *
 tor_tls_new(int sock, int isServer, int use_no_cert)
 {
-  tor_tls *result = tor_malloc(sizeof(tor_tls));
+  tor_tls_t *result = tor_malloc(sizeof(tor_tls_t));
   SSL_CTX *ctx;
   tor_assert(global_tls_context); /* make sure somebody made it first */
   ctx = use_no_cert ? global_tls_context->client_only_ctx
@@ -439,7 +460,7 @@ tor_tls_new(int sock, int isServer, int use_no_cert)
 /** Return whether this tls initiated the connect (client) or
  * received it (server). */
 int
-tor_tls_is_server(tor_tls *tls)
+tor_tls_is_server(tor_tls_t *tls)
 {
   tor_assert(tls);
   return tls->isServer;
@@ -449,7 +470,7 @@ tor_tls_is_server(tor_tls *tls)
  * underlying file descriptor.
  */
 void
-tor_tls_free(tor_tls *tls)
+tor_tls_free(tor_tls_t *tls)
 {
   tor_assert(tls && tls->ssl);
   SSL_free(tls->ssl);
@@ -463,7 +484,7 @@ tor_tls_free(tor_tls *tls)
  * TOR_TLS_CLOSE, TOR_TLS_WANTREAD, or TOR_TLS_WANTWRITE.
  */
 int
-tor_tls_read(tor_tls *tls, char *cp, size_t len)
+tor_tls_read(tor_tls_t *tls, char *cp, size_t len)
 {
   int r, err;
   tor_assert(tls);
@@ -472,14 +493,14 @@ tor_tls_read(tor_tls *tls, char *cp, size_t len)
   r = SSL_read(tls->ssl, cp, len);
   if (r > 0)
     return r;
-  err = tor_tls_get_error(tls, r, CATCH_ZERO, "reading", LOG_INFO);
+  err = tor_tls_get_error(tls, r, CATCH_ZERO, "reading", LOG_DEBUG);
   if (err == _TOR_TLS_ZERORETURN) {
-    log_fn(LOG_DEBUG,"read returned r=%d; TLS is closed",r);
+    log_debug(LD_NET,"read returned r=%d; TLS is closed",r);
     tls->state = TOR_TLS_ST_CLOSED;
     return TOR_TLS_CLOSE;
   } else {
     tor_assert(err != TOR_TLS_DONE);
-    log_fn(LOG_DEBUG,"read returned r=%d, err=%d",r,err);
+    log_debug(LD_NET,"read returned r=%d, err=%d",r,err);
     return err;
   }
 }
@@ -490,7 +511,7 @@ tor_tls_read(tor_tls *tls, char *cp, size_t len)
  * TOR_TLS_WANTREAD, or TOR_TLS_WANTWRITE.
  */
 int
-tor_tls_write(tor_tls *tls, char *cp, size_t n)
+tor_tls_write(tor_tls_t *tls, char *cp, size_t n)
 {
   int r, err;
   tor_assert(tls);
@@ -501,8 +522,8 @@ tor_tls_write(tor_tls *tls, char *cp, size_t n)
   if (tls->wantwrite_n) {
     /* if WANTWRITE last time, we must use the _same_ n as before */
     tor_assert(n >= tls->wantwrite_n);
-    log_fn(LOG_DEBUG,"resuming pending-write, (%d to flush, reusing %d)",
-           (int)n, (int)tls->wantwrite_n);
+    log_debug(LD_NET,"resuming pending-write, (%d to flush, reusing %d)",
+              (int)n, (int)tls->wantwrite_n);
     n = tls->wantwrite_n;
     tls->wantwrite_n = 0;
   }
@@ -522,7 +543,7 @@ tor_tls_write(tor_tls *tls, char *cp, size_t n)
  * or TOR_TLS_WANTWRITE.
  */
 int
-tor_tls_handshake(tor_tls *tls)
+tor_tls_handshake(tor_tls_t *tls)
 {
   int r;
   tor_assert(tls);
@@ -550,7 +571,7 @@ tor_tls_handshake(tor_tls *tls)
  * or TOR_TLS_WANTWRITE.
  */
 int
-tor_tls_shutdown(tor_tls *tls)
+tor_tls_shutdown(tor_tls_t *tls)
 {
   int r, err;
   char buf[128];
@@ -595,7 +616,7 @@ tor_tls_shutdown(tor_tls *tls)
        */
       if (tls->state == TOR_TLS_ST_GOTCLOSE ||
          tls->state == TOR_TLS_ST_SENTCLOSE) {
-        log(LOG_WARN,
+        log(LOG_WARN, LD_NET,
             "TLS returned \"half-closed\" value while already half-closed");
         return TOR_TLS_ERROR;
       }
@@ -610,7 +631,7 @@ tor_tls_shutdown(tor_tls *tls)
 /** Return true iff this TLS connection is authenticated.
  */
 int
-tor_tls_peer_has_cert(tor_tls *tls)
+tor_tls_peer_has_cert(tor_tls_t *tls)
 {
   X509 *cert;
   cert = SSL_get_peer_certificate(tls->ssl);
@@ -627,7 +648,7 @@ tor_tls_peer_has_cert(tor_tls *tls)
  * NUL-terminate.  Return 0 on success, -1 on failure.
  */
 int
-tor_tls_get_peer_cert_nickname(tor_tls *tls, char *buf, size_t buflen)
+tor_tls_get_peer_cert_nickname(tor_tls_t *tls, char *buf, size_t buflen)
 {
   X509 *cert = NULL;
   X509_NAME *name = NULL;
@@ -636,11 +657,11 @@ tor_tls_get_peer_cert_nickname(tor_tls *tls, char *buf, size_t buflen)
   int r = -1;
 
   if (!(cert = SSL_get_peer_certificate(tls->ssl))) {
-    log_fn(LOG_WARN, "Peer has no certificate");
+    log_warn(LD_PROTOCOL, "Peer has no certificate");
     goto error;
   }
   if (!(name = X509_get_subject_name(cert))) {
-    log_fn(LOG_WARN, "Peer certificate has no subject name");
+    log_warn(LD_PROTOCOL, "Peer certificate has no subject name");
     goto error;
   }
   if ((nid = OBJ_txt2nid("commonName")) == NID_undef)
@@ -650,10 +671,12 @@ tor_tls_get_peer_cert_nickname(tor_tls *tls, char *buf, size_t buflen)
   if (lenout == -1)
     goto error;
   if (((int)strspn(buf, LEGAL_NICKNAME_CHARACTERS)) < lenout) {
-    log_fn(LOG_WARN, "Peer certificate nickname '%s' has illegal characters.",
-           buf);
+    log_warn(LD_PROTOCOL,
+             "Peer certificate nickname %s has illegal characters.",
+             escaped(buf));
     if (strchr(buf, '.'))
-      log_fn(LOG_WARN, "  (Maybe it is not really running Tor at its advertised OR port.)");
+      log_warn(LD_PROTOCOL, "  (Maybe it is not really running Tor at its "
+               "advertised OR port.)");
     goto error;
   }
 
@@ -667,7 +690,8 @@ tor_tls_get_peer_cert_nickname(tor_tls *tls, char *buf, size_t buflen)
   return r;
 }
 
-static void log_cert_lifetime(X509 *cert, const char *problem)
+static void
+log_cert_lifetime(X509 *cert, const char *problem)
 {
   BIO *bio = NULL;
   BUF_MEM *buf;
@@ -677,11 +701,12 @@ static void log_cert_lifetime(X509 *cert, const char *problem)
   struct tm tm;
 
   if (problem)
-    log_fn(LOG_WARN,"Certificate %s: is your system clock set incorrectly?",
-           problem);
+    log_warn(LD_GENERAL,
+             "Certificate %s: is your system clock set incorrectly?",
+             problem);
 
   if (!(bio = BIO_new(BIO_s_mem()))) {
-    log_fn(LOG_WARN, "Couldn't allocate BIO!"); goto end;
+    log_warn(LD_GENERAL, "Couldn't allocate BIO!"); goto end;
   }
   if (!(ASN1_TIME_print(bio, X509_get_notBefore(cert)))) {
     tls_log_errors(LOG_WARN, "printing certificate lifetime");
@@ -700,7 +725,9 @@ static void log_cert_lifetime(X509 *cert, const char *problem)
 
   strftime(mytime, 32, "%b %d %H:%M:%S %Y GMT", tor_gmtime_r(&now, &tm));
 
-  log_fn(LOG_WARN, "(certificate lifetime runs from %s through %s. Your time is %s.)",s1,s2,mytime);
+  log_warn(LD_GENERAL,
+           "(certificate lifetime runs from %s through %s. Your time is %s.)",
+           s1,s2,mytime);
 
  end:
   /* Not expected to get invoked */
@@ -716,10 +743,10 @@ static void log_cert_lifetime(X509 *cert, const char *problem)
 /** If the provided tls connection is authenticated and has a
  * certificate that is currently valid and signed, then set
  * *<b>identity_key</b> to the identity certificate's key and return
- * 0.  Else, return -1.
+ * 0.  Else, return -1 and log complaints with log-level <b>severity</b>.
  */
 int
-tor_tls_verify(tor_tls *tls, crypto_pk_env_t **identity_key)
+tor_tls_verify(int severity, tor_tls_t *tls, crypto_pk_env_t **identity_key)
 {
   X509 *cert = NULL, *id_cert = NULL;
   STACK_OF(X509) *chain = NULL;
@@ -740,7 +767,8 @@ tor_tls_verify(tor_tls *tls, crypto_pk_env_t **identity_key)
    * cert and the id_cert.
    */
   if (num_in_chain < 1) {
-    log_fn(LOG_WARN,"Unexpected number of certificates in chain (%d)",
+    log_fn(severity,LD_PROTOCOL,
+           "Unexpected number of certificates in chain (%d)",
            num_in_chain);
     goto done;
   }
@@ -750,14 +778,14 @@ tor_tls_verify(tor_tls *tls, crypto_pk_env_t **identity_key)
       break;
   }
   if (!id_cert) {
-    log_fn(LOG_WARN,"No distinct identity certificate found");
+    log_fn(severity,LD_PROTOCOL,"No distinct identity certificate found");
     goto done;
   }
 
   if (!(id_pkey = X509_get_pubkey(id_cert)) ||
       X509_verify(cert, id_pkey) <= 0) {
-    log_fn(LOG_WARN,"X509_verify on cert and pkey returned <= 0");
-    tls_log_errors(LOG_WARN,"verifying certificate");
+    log_fn(severity,LD_PROTOCOL,"X509_verify on cert and pkey returned <= 0");
+    tls_log_errors(severity,"verifying certificate");
     goto done;
   }
 
@@ -788,7 +816,7 @@ tor_tls_verify(tor_tls *tls, crypto_pk_env_t **identity_key)
  * NOTE: you should call tor_tls_verify before tor_tls_check_lifetime.
  */
 int
-tor_tls_check_lifetime(tor_tls *tls, int tolerance)
+tor_tls_check_lifetime(tor_tls_t *tls, int tolerance)
 {
   time_t now, t;
   X509 *cert;
@@ -823,7 +851,7 @@ tor_tls_check_lifetime(tor_tls *tls, int tolerance)
 /** Return the number of bytes available for reading from <b>tls</b>.
  */
 int
-tor_tls_get_pending_bytes(tor_tls *tls)
+tor_tls_get_pending_bytes(tor_tls_t *tls)
 {
   tor_assert(tls);
 #if OPENSSL_VERSION_NUMBER < 0x0090700fl
@@ -837,13 +865,15 @@ tor_tls_get_pending_bytes(tor_tls *tls)
 }
 
 /** Return the number of bytes read across the underlying socket. */
-unsigned long tor_tls_get_n_bytes_read(tor_tls *tls)
+unsigned long
+tor_tls_get_n_bytes_read(tor_tls_t *tls)
 {
   tor_assert(tls);
   return BIO_number_read(SSL_get_rbio(tls->ssl));
 }
 /** Return the number of bytes written across the underlying socket. */
-unsigned long tor_tls_get_n_bytes_written(tor_tls *tls)
+unsigned long
+tor_tls_get_n_bytes_written(tor_tls_t *tls)
 {
   tor_assert(tls);
   return BIO_number_written(SSL_get_wbio(tls->ssl));
@@ -851,12 +881,13 @@ unsigned long tor_tls_get_n_bytes_written(tor_tls *tls)
 
 /** Implement check_no_tls_errors: If there are any pending OpenSSL
  * errors, log an error message and assert(0). */
-void _check_no_tls_errors(const char *fname, int line)
+void
+_check_no_tls_errors(const char *fname, int line)
 {
   if (ERR_peek_error() == 0)
     return;
-  log_fn(LOG_WARN, "Unhandled OpenSSL errors found at %s:%d: ",
-         fname, line);
+  log(LOG_WARN, LD_CRYPTO, "Unhandled OpenSSL errors found at %s:%d: ",
+      tor_fix_source_file(fname), line);
   tls_log_errors(LOG_WARN, NULL);
 }
 
