@@ -1,13 +1,15 @@
 /* Copyright 2003-2004 Roger Dingledine.
- * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+ * Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: cpuworker.c,v 1.79.2.1 2005/09/23 01:24:09 arma Exp $ */
-const char cpuworker_c_id[] = "$Id: cpuworker.c,v 1.79.2.1 2005/09/23 01:24:09 arma Exp $";
+/* $Id: cpuworker.c,v 1.98 2006/02/13 09:02:35 arma Exp $ */
+const char cpuworker_c_id[] =
+  "$Id: cpuworker.c,v 1.98 2006/02/13 09:02:35 arma Exp $";
 
 /**
  * \file cpuworker.c
- * \brief Run computation-intensive tasks (generally for crypto) in
- * a separate execution context. [OR only.]
+ * \brief Implements a farm of 'CPU worker' processes to perform
+ * CPU-intensive tasks in another thread or process, to not
+ * interrupt the main thread.
  *
  * Right now, we only use this for processing onionskins.
  **/
@@ -24,7 +26,8 @@ const char cpuworker_c_id[] = "$Id: cpuworker.c,v 1.79.2.1 2005/09/23 01:24:09 a
 /** How many bytes are sent from tor to the cpuworker? */
 #define LEN_ONION_QUESTION (1+TAG_LEN+ONIONSKIN_CHALLENGE_LEN)
 /** How many bytes are sent from the cpuworker back to tor? */
-#define LEN_ONION_RESPONSE (1+TAG_LEN+ONIONSKIN_REPLY_LEN+CPATH_KEY_MATERIAL_LEN)
+#define LEN_ONION_RESPONSE \
+  (1+TAG_LEN+ONIONSKIN_REPLY_LEN+CPATH_KEY_MATERIAL_LEN)
 
 /** How many cpuworkers we have running right now. */
 static int num_cpuworkers=0;
@@ -42,13 +45,16 @@ static void process_pending_task(connection_t *cpuworker);
 
 /** Initialize the cpuworker subsystem.
  */
-void cpu_init(void) {
-  last_rotation_time=time(NULL);
-  spawn_enough_cpuworkers();
+void
+cpu_init(void)
+{
+  cpuworkers_rotate();
 }
 
 /** Called when we're done sending a request to a cpuworker. */
-int connection_cpu_finished_flushing(connection_t *conn) {
+int
+connection_cpu_finished_flushing(connection_t *conn)
+{
   tor_assert(conn);
   tor_assert(conn->type == CONN_TYPE_CPUWORKER);
   connection_stop_writing(conn);
@@ -57,7 +63,9 @@ int connection_cpu_finished_flushing(connection_t *conn) {
 
 /** Pack addr,port,and circ_id; set *tag to the result. (See note on
  * cpuworker_main for wire format.) */
-static void tag_pack(char *tag, uint32_t addr, uint16_t port, uint16_t circ_id) {
+static void
+tag_pack(char *tag, uint32_t addr, uint16_t port, uint16_t circ_id)
+{
   *(uint32_t *)tag     = addr;
   *(uint16_t *)(tag+4) = port;
   *(uint16_t *)(tag+6) = circ_id;
@@ -65,7 +73,9 @@ static void tag_pack(char *tag, uint32_t addr, uint16_t port, uint16_t circ_id)
 
 /** Unpack <b>tag</b> into addr, port, and circ_id.
  */
-static void tag_unpack(const char *tag, uint32_t *addr, uint16_t *port, uint16_t *circ_id) {
+static void
+tag_unpack(const char *tag, uint32_t *addr, uint16_t *port, uint16_t *circ_id)
+{
   struct in_addr in;
   char addrbuf[INET_NTOA_BUF_LEN];
 
@@ -75,14 +85,16 @@ static void tag_unpack(const char *tag, uint32_t *addr, uint16_t *port, uint16_t
 
   in.s_addr = htonl(*addr);
   tor_inet_ntoa(&in, addrbuf, sizeof(addrbuf));
-  log_fn(LOG_DEBUG,"onion was from %s:%d, circ_id %d.", addrbuf, *port, *circ_id);
+  log_debug(LD_OR,
+            "onion was from %s:%d, circ_id %d.", addrbuf, *port, *circ_id);
 }
 
 /** Called when the onion key has changed and we need to spawn new
  * cpuworkers.  Close all currently idle cpuworkers, and mark the last
  * rotation time as now.
  */
-void cpuworkers_rotate(void)
+void
+cpuworkers_rotate(void)
 {
   connection_t *cpuworker;
   while ((cpuworker = connection_get_by_type_state(CONN_TYPE_CPUWORKER,
@@ -91,22 +103,26 @@ void cpuworkers_rotate(void)
     --num_cpuworkers;
   }
   last_rotation_time = time(NULL);
-  spawn_enough_cpuworkers();
+  if (server_mode(get_options()))
+    spawn_enough_cpuworkers();
 }
 
 /** If the cpuworker closes the connection,
  * mark it as closed and spawn a new one as needed. */
-int connection_cpu_reached_eof(connection_t *conn) {
-  log_fn(LOG_WARN,"Read eof. Worker died unexpectedly.");
+int
+connection_cpu_reached_eof(connection_t *conn)
+{
+  log_warn(LD_GENERAL,"Read eof. Worker died unexpectedly.");
   if (conn->state != CPUWORKER_STATE_IDLE) {
     /* the circ associated with this cpuworker will have to wait until
      * it gets culled in run_connection_housekeeping(), since we have
      * no way to find out which circ it was. */
-    log_fn(LOG_WARN,"...and it left a circuit queued; abandoning circ.");
+    log_warn(LD_GENERAL,"...and it left a circuit queued; abandoning circ.");
     num_cpuworkers_busy--;
   }
   num_cpuworkers--;
-  spawn_enough_cpuworkers(); /* try to regrow. hope we don't end up spinning. */
+  spawn_enough_cpuworkers(); /* try to regrow. hope we don't end up
+                                spinning. */
   connection_mark_for_close(conn);
   return 0;
 }
@@ -115,7 +131,9 @@ int connection_cpu_reached_eof(connection_t *conn) {
  * wait for a complete answer. If the answer is complete,
  * process it as appropriate.
  */
-int connection_cpu_process_inbuf(connection_t *conn) {
+int
+connection_cpu_process_inbuf(connection_t *conn)
+{
   char success;
   char buf[LEN_ONION_RESPONSE];
   uint32_t addr;
@@ -131,7 +149,7 @@ int connection_cpu_process_inbuf(connection_t *conn) {
     return 0;
 
   if (conn->state == CPUWORKER_STATE_BUSY_ONION) {
-    if (buf_datalen(conn->inbuf) < LEN_ONION_RESPONSE) /* entire answer available? */
+    if (buf_datalen(conn->inbuf) < LEN_ONION_RESPONSE) /* answer available? */
       return 0; /* not yet */
     tor_assert(buf_datalen(conn->inbuf) == LEN_ONION_RESPONSE);
 
@@ -149,22 +167,30 @@ int connection_cpu_process_inbuf(connection_t *conn) {
       circ = circuit_get_by_circid_orconn(circ_id, p_conn);
 
     if (success == 0) {
-      log_fn(LOG_INFO,"decoding onionskin failed. Closing.");
+      log_debug(LD_OR,
+                "decoding onionskin failed. "
+                "(Old key or bad software.) Closing.");
       if (circ)
-        circuit_mark_for_close(circ);
+        circuit_mark_for_close(circ, END_CIRC_REASON_TORPROTOCOL);
       goto done_processing;
     }
     if (!circ) {
-      log_fn(LOG_INFO,"processed onion for a circ that's gone. Dropping.");
+      /* This happens because somebody sends us a destroy cell and the
+       * circuit goes away, while the cpuworker is working. This is also
+       * why our tag doesn't include a pointer to the circ, because we'd
+       * never know if it's still valid.
+       */
+      log_debug(LD_OR,"processed onion for a circ that's gone. Dropping.");
       goto done_processing;
     }
     tor_assert(circ->p_conn);
-    if (onionskin_answer(circ, CELL_CREATED, buf+TAG_LEN, buf+TAG_LEN+ONIONSKIN_REPLY_LEN) < 0) {
-      log_fn(LOG_WARN,"onionskin_answer failed. Closing.");
-      circuit_mark_for_close(circ);
+    if (onionskin_answer(circ, CELL_CREATED, buf+TAG_LEN,
+                         buf+TAG_LEN+ONIONSKIN_REPLY_LEN) < 0) {
+      log_warn(LD_OR,"onionskin_answer failed. Closing.");
+      circuit_mark_for_close(circ, END_CIRC_REASON_INTERNAL);
       goto done_processing;
     }
-    log_fn(LOG_DEBUG,"onionskin_answer succeeded. Yay.");
+    log_debug(LD_OR,"onionskin_answer succeeded. Yay.");
   } else {
     tor_assert(0); /* don't ask me to do handshakes yet */
   }
@@ -198,7 +224,9 @@ done_processing:
  *  (Note: this _should_ be by addr/port, since we're concerned with specific
  * connections, not with routers (where we'd use identity).)
  */
-static int cpuworker_main(void *data) {
+static int
+cpuworker_main(void *data)
+{
   char question[ONIONSKIN_CHALLENGE_LEN];
   uint8_t question_type;
   int *fdarray = data;
@@ -213,7 +241,8 @@ static int cpuworker_main(void *data) {
 
   fd = fdarray[1]; /* this side is ours */
 #ifndef TOR_IS_MULTITHREADED
-  tor_close_socket(fdarray[0]); /* this is the side of the socketpair the parent uses */
+  tor_close_socket(fdarray[0]); /* this is the side of the socketpair the
+                                 * parent uses */
   tor_free_all(1); /* so the child doesn't hold the parent's fd's open */
   handle_signals(0); /* ignore interrupts from the keyboard, etc */
 #endif
@@ -227,22 +256,28 @@ static int cpuworker_main(void *data) {
     if ((r = recv(fd, &question_type, 1, 0)) != 1) {
 //      log_fn(LOG_ERR,"read type failed. Exiting.");
       if (r == 0) {
-        log_fn(LOG_INFO,"CPU worker exiting because Tor process closed connection (either rotated keys or died).");
+        log_info(LD_OR,
+                 "CPU worker exiting because Tor process closed connection "
+                 "(either rotated keys or died).");
       } else {
-        log_fn(LOG_INFO,"CPU worker editing because of error on connection to Tor process.");
-        log_fn(LOG_INFO,"(Error on %d was %s)", fd, tor_socket_strerror(tor_socket_errno(fd)));
+        log_info(LD_OR,
+                 "CPU worker editing because of error on connection to Tor "
+                 "process.");
+        log_info(LD_OR,"(Error on %d was %s)",
+                 fd, tor_socket_strerror(tor_socket_errno(fd)));
       }
       goto end;
     }
     tor_assert(question_type == CPUWORKER_TASK_ONION);
 
     if (read_all(fd, tag, TAG_LEN, 1) != TAG_LEN) {
-      log_fn(LOG_ERR,"read tag failed. Exiting.");
+      log_err(LD_BUG,"read tag failed. Exiting.");
       goto end;
     }
 
-    if (read_all(fd, question, ONIONSKIN_CHALLENGE_LEN, 1) != ONIONSKIN_CHALLENGE_LEN) {
-      log_fn(LOG_ERR,"read question failed. Exiting.");
+    if (read_all(fd, question, ONIONSKIN_CHALLENGE_LEN, 1) !=
+        ONIONSKIN_CHALLENGE_LEN) {
+      log_err(LD_BUG,"read question failed. Exiting.");
       goto end;
     }
 
@@ -250,21 +285,21 @@ static int cpuworker_main(void *data) {
       if (onion_skin_server_handshake(question, onion_key, last_onion_key,
           reply_to_proxy, keys, CPATH_KEY_MATERIAL_LEN) < 0) {
         /* failure */
-        log_fn(LOG_INFO,"onion_skin_server_handshake failed.");
+        log_debug(LD_OR,"onion_skin_server_handshake failed.");
         memset(buf,0,LEN_ONION_RESPONSE); /* send all zeros for failure */
       } else {
         /* success */
-        log_fn(LOG_DEBUG,"onion_skin_server_handshake succeeded.");
+        log_debug(LD_OR,"onion_skin_server_handshake succeeded.");
         buf[0] = 1; /* 1 means success */
         memcpy(buf+1,tag,TAG_LEN);
         memcpy(buf+1+TAG_LEN,reply_to_proxy,ONIONSKIN_REPLY_LEN);
         memcpy(buf+1+TAG_LEN+ONIONSKIN_REPLY_LEN,keys,CPATH_KEY_MATERIAL_LEN);
       }
       if (write_all(fd, buf, LEN_ONION_RESPONSE, 1) != LEN_ONION_RESPONSE) {
-        log_fn(LOG_ERR,"writing response buf failed. Exiting.");
+        log_err(LD_BUG,"writing response buf failed. Exiting.");
         goto end;
       }
-      log_fn(LOG_DEBUG,"finished writing response.");
+      log_debug(LD_OR,"finished writing response.");
     }
   }
  end:
@@ -273,30 +308,34 @@ static int cpuworker_main(void *data) {
   if (last_onion_key)
     crypto_free_pk_env(last_onion_key);
   tor_close_socket(fd);
+  crypto_thread_cleanup();
   spawn_exit();
   return 0; /* windows wants this function to return an int */
 }
 
 /** Launch a new cpuworker. Return 0 if we're happy, -1 if we failed.
  */
-static int spawn_cpuworker(void) {
+static int
+spawn_cpuworker(void)
+{
   int *fdarray;
   int fd;
   connection_t *conn;
+  int err;
 
   fdarray = tor_malloc(sizeof(int)*2);
-  if (tor_socketpair(AF_UNIX, SOCK_STREAM, 0, fdarray) < 0) {
-    log(LOG_ERR, "Couldn't construct socketpair: %s",
-        tor_socket_strerror(tor_socket_errno(-1)));
+  if ((err = tor_socketpair(AF_UNIX, SOCK_STREAM, 0, fdarray)) < 0) {
+    log_warn(LD_NET, "Couldn't construct socketpair: %s",
+             tor_socket_strerror(-err));
     tor_free(fdarray);
     return -1;
   }
 
   fd = fdarray[0];
   spawn_func(cpuworker_main, (void*)fdarray);
-  log_fn(LOG_DEBUG,"just spawned a worker.");
+  log_debug(LD_OR,"just spawned a cpu worker.");
 #ifndef TOR_IS_MULTITHREADED
-  tor_close_socket(fdarray[1]); /* we don't need the worker's side of the pipe */
+  tor_close_socket(fdarray[1]); /* don't need the worker's side of the pipe */
   tor_free(fdarray);
 #endif
 
@@ -309,7 +348,7 @@ static int spawn_cpuworker(void) {
   conn->address = tor_strdup("localhost");
 
   if (connection_add(conn) < 0) { /* no space, forget it */
-    log_fn(LOG_WARN,"connection_add failed. Giving up.");
+    log_warn(LD_NET,"connection_add failed. Giving up.");
     connection_free(conn); /* this closes fd */
     return -1;
   }
@@ -323,7 +362,9 @@ static int spawn_cpuworker(void) {
 /** If we have too few or too many active cpuworkers, try to spawn new ones
  * or kill idle ones.
  */
-static void spawn_enough_cpuworkers(void) {
+static void
+spawn_enough_cpuworkers(void)
+{
   int num_cpuworkers_needed = get_options()->NumCpus;
 
   if (num_cpuworkers_needed < MIN_CPUWORKERS)
@@ -333,7 +374,7 @@ static void spawn_enough_cpuworkers(void) {
 
   while (num_cpuworkers < num_cpuworkers_needed) {
     if (spawn_cpuworker() < 0) {
-      log_fn(LOG_WARN,"Spawn failed. Will try again later.");
+      log_warn(LD_GENERAL,"Spawn failed. Will try again later.");
       return;
     }
     num_cpuworkers++;
@@ -341,7 +382,9 @@ static void spawn_enough_cpuworkers(void) {
 }
 
 /** Take a pending task from the queue and assign it to 'cpuworker'. */
-static void process_pending_task(connection_t *cpuworker) {
+static void
+process_pending_task(connection_t *cpuworker)
+{
   circuit_t *circ;
 
   tor_assert(cpuworker);
@@ -352,7 +395,7 @@ static void process_pending_task(connection_t *cpuworker) {
   if (!circ)
     return;
   if (assign_to_cpuworker(cpuworker, CPUWORKER_TASK_ONION, circ) < 0)
-    log_fn(LOG_WARN,"assign_to_cpuworker failed. Ignoring.");
+    log_warn(LD_OR,"assign_to_cpuworker failed. Ignoring.");
 }
 
 #define CPUWORKER_BUSY_TIMEOUT 3600 /* seconds */
@@ -362,7 +405,8 @@ static void process_pending_task(connection_t *cpuworker) {
  * thinks of itself as idle. I don't know why. But here's a workaround
  * to kill any cpuworker that's been busy for more than 3600 seconds. */
 static void
-cull_wedged_cpuworkers(void) {
+cull_wedged_cpuworkers(void)
+{
   connection_t **carray;
   connection_t *conn;
   int n_conns, i;
@@ -375,7 +419,8 @@ cull_wedged_cpuworkers(void) {
         conn->type == CONN_TYPE_CPUWORKER &&
         conn->state == CPUWORKER_STATE_BUSY_ONION &&
         conn->timestamp_lastwritten + CPUWORKER_BUSY_TIMEOUT < now) {
-      log_fn(LOG_NOTICE,"Bug: closing wedged cpuworker. Can somebody find the bug?");
+      log_notice(LD_BUG,
+                 "Bug: closing wedged cpuworker. Can somebody find the bug?");
       num_cpuworkers_busy--;
       num_cpuworkers--;
       connection_mark_for_close(conn);
@@ -389,8 +434,10 @@ cull_wedged_cpuworkers(void) {
  * If question_type is CPUWORKER_TASK_ONION then task is a circ.
  * No other question_types are allowed.
  */
-int assign_to_cpuworker(connection_t *cpuworker, uint8_t question_type,
-                        void *task) {
+int
+assign_to_cpuworker(connection_t *cpuworker, uint8_t question_type,
+                    void *task)
+{
   circuit_t *circ;
   char tag[TAG_LEN];
 
@@ -401,21 +448,23 @@ int assign_to_cpuworker(connection_t *cpuworker, uint8_t question_type,
 
   if (question_type == CPUWORKER_TASK_ONION) {
     circ = task;
+    tor_assert(circ->onionskin);
 
     if (num_cpuworkers_busy == num_cpuworkers) {
-      log_fn(LOG_DEBUG,"No idle cpuworkers. Queuing.");
+      log_debug(LD_OR,"No idle cpuworkers. Queuing.");
       if (onion_pending_add(circ) < 0)
         return -1;
       return 0;
     }
 
     if (!cpuworker)
-      cpuworker = connection_get_by_type_state(CONN_TYPE_CPUWORKER, CPUWORKER_STATE_IDLE);
+      cpuworker = connection_get_by_type_state(CONN_TYPE_CPUWORKER,
+                                               CPUWORKER_STATE_IDLE);
 
     tor_assert(cpuworker);
 
     if (!circ->p_conn) {
-      log_fn(LOG_INFO,"circ->p_conn gone. Failing circ.");
+      log_info(LD_OR,"circ->p_conn gone. Failing circ.");
       return -1;
     }
     tag_pack(tag, circ->p_conn->addr, circ->p_conn->port, circ->p_circ_id);
@@ -425,7 +474,9 @@ int assign_to_cpuworker(connection_t *cpuworker, uint8_t question_type,
 
     connection_write_to_buf((char*)&question_type, 1, cpuworker);
     connection_write_to_buf(tag, sizeof(tag), cpuworker);
-    connection_write_to_buf(circ->onionskin, ONIONSKIN_CHALLENGE_LEN, cpuworker);
+    connection_write_to_buf(circ->onionskin, ONIONSKIN_CHALLENGE_LEN,
+                            cpuworker);
+    tor_free(circ->onionskin);
   }
   return 0;
 }
