@@ -1,7 +1,8 @@
-/* Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
+/* Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: hibernate.c,v 1.51 2005/04/02 22:11:24 nickm Exp $ */
-const char hibernate_c_id[] = "$Id: hibernate.c,v 1.51 2005/04/02 22:11:24 nickm Exp $";
+/* $Id: hibernate.c,v 1.68 2006/02/13 09:37:53 arma Exp $ */
+const char hibernate_c_id[] =
+  "$Id: hibernate.c,v 1.68 2006/02/13 09:37:53 arma Exp $";
 
 /**
  * \file hibernate.c
@@ -128,7 +129,7 @@ accounting_parse_options(or_options_t *options, int validate_only)
   smartlist_split_string(items, v, NULL,
                          SPLIT_SKIP_SPACE|SPLIT_IGNORE_BLANK,0);
   if (smartlist_len(items)<2) {
-    log_fn(LOG_WARN, "Too few arguments to AccountingStart");
+    log_warn(LD_CONFIG, "Too few arguments to AccountingStart");
     goto err;
   }
   s = smartlist_get(items,0);
@@ -139,7 +140,9 @@ accounting_parse_options(or_options_t *options, int validate_only)
   } else if (0==strcasecmp(s, "day")) {
     unit = UNIT_DAY;
   } else {
-    log_fn(LOG_WARN, "Unrecognized accounting unit '%s': only 'month', 'week', and 'day' are supported.", s);
+    log_warn(LD_CONFIG,
+             "Unrecognized accounting unit '%s': only 'month', 'week',"
+             " and 'day' are supported.", s);
     goto err;
   }
 
@@ -147,14 +150,16 @@ accounting_parse_options(or_options_t *options, int validate_only)
   case UNIT_WEEK:
     d = tor_parse_long(smartlist_get(items,1), 10, 1, 7, &ok, NULL);
     if (!ok) {
-      log_fn(LOG_WARN, "Weekly accounting must start begin on a day between 1(Monday) and 7 (Sunday)");
+      log_warn(LD_CONFIG, "Weekly accounting must begin on a day between "
+               "1 (Monday) and 7 (Sunday)");
       goto err;
     }
     break;
   case UNIT_MONTH:
     d = tor_parse_long(smartlist_get(items,1), 10, 1, 28, &ok, NULL);
     if (!ok) {
-      log_fn(LOG_WARN, "Monthly accounting must start begin on a day between 1 and 28");
+      log_warn(LD_CONFIG, "Monthly accounting must begin on a day between "
+               "1 and 28");
       goto err;
     }
     break;
@@ -167,27 +172,29 @@ accounting_parse_options(or_options_t *options, int validate_only)
 
   idx = unit==UNIT_DAY?1:2;
   if (smartlist_len(items) != (idx+1)) {
-    log_fn(LOG_WARN, "Accounting unit '%s' requires %d arguments",
-           s, idx+1);
+    log_warn(LD_CONFIG,"Accounting unit '%s' requires %d argument%s.",
+             s, idx, (idx>1)?"s":"");
     goto err;
   }
   s = smartlist_get(items, idx);
   h = tor_parse_long(s, 10, 0, 23, &ok, &cp);
   if (!ok) {
-    log_fn(LOG_WARN, "Accounting start time not parseable: bad hour.");
+    log_warn(LD_CONFIG,"Accounting start time not parseable: bad hour.");
     goto err;
   }
   if (!cp || *cp!=':') {
-    log_fn(LOG_WARN,"Accounting start time not parseable: not in HH:MM format");
+    log_warn(LD_CONFIG,
+             "Accounting start time not parseable: not in HH:MM format");
     goto err;
   }
   m = tor_parse_long(cp+1, 10, 0, 59, &ok, &cp);
   if (!ok) {
-    log_fn(LOG_WARN, "Accounting start time not parseable: bad minute");
+    log_warn(LD_CONFIG, "Accounting start time not parseable: bad minute");
     goto err;
   }
   if (!cp || *cp!='\0') {
-    log_fn(LOG_WARN,"Accounting start time not parseable: not in HH:MM format");
+    log_warn(LD_CONFIG,
+             "Accounting start time not parseable: not in HH:MM format");
     goto err;
   }
 
@@ -209,7 +216,9 @@ accounting_parse_options(or_options_t *options, int validate_only)
 /** If we want to manage the accounting system and potentially
  * hibernate, return 1, else return 0.
  */
-int accounting_is_enabled(or_options_t *options) {
+int
+accounting_is_enabled(or_options_t *options)
+{
   if (options->AccountingMax)
     return 1;
   return 0;
@@ -319,15 +328,16 @@ configure_accounting(time_t now)
       start_of_accounting_period_after(interval_start_time) <= now) {
     /* We didn't have recorded usage, or we don't have recorded usage
      * for this interval. Start a new interval. */
-    log_fn(LOG_INFO, "Starting new accounting interval.");
+    log_info(LD_ACCT, "Starting new accounting interval.");
     reset_accounting(now);
   } else if (interval_start_time ==
         start_of_accounting_period_containing(interval_start_time)) {
-    log_fn(LOG_INFO, "Continuing accounting interval.");
+    log_info(LD_ACCT, "Continuing accounting interval.");
     /* We are in the interval we thought we were in. Do nothing.*/
     interval_end_time = start_of_accounting_period_after(interval_start_time);
   } else {
-    log_fn(LOG_WARN, "Mismatched accounting interval; starting a fresh one.");
+    log_warn(LD_ACCT,
+             "Mismatched accounting interval; starting a fresh one.");
     reset_accounting(now);
   }
   accounting_set_wakeup_time();
@@ -366,8 +376,9 @@ update_expected_bandwidth(void)
  * the start and end of the interval, and clear byte/time totals.
  */
 static void
-reset_accounting(time_t now) {
-  log_fn(LOG_INFO, "Starting new accounting interval.");
+reset_accounting(time_t now)
+{
+  log_info(LD_ACCT, "Starting new accounting interval.");
   update_expected_bandwidth();
   interval_start_time = start_of_accounting_period_containing(now);
   interval_end_time = start_of_accounting_period_after(interval_start_time);
@@ -400,6 +411,8 @@ time_to_record_bandwidth_usage(time_t now)
   return 0;
 }
 
+/** Invoked once per second.  Checks whether it is time to hibernate,
+ * record bandwidth used, etc.  */
 void
 accounting_run_housekeeping(time_t now)
 {
@@ -408,10 +421,10 @@ accounting_run_housekeeping(time_t now)
   }
   if (time_to_record_bandwidth_usage(now)) {
     if (accounting_record_bandwidth_usage(now)) {
-      log_fn(LOG_ERR, "Couldn't record bandwidth usage; exiting.");
-      /* XXX this can fail when you're out of fd's, causing a crash.
-       * Perhaps the better answer is to hold the file open all the
-       * time? */
+      log_err(LD_FS, "Couldn't record bandwidth usage to disk; exiting.");
+      /* This can fail when we're out of fd's, causing a crash.
+       * The current answer is to reserve 32 more than we need, in
+       * set_max_file_descriptors(). */
       exit(1);
     }
   }
@@ -431,7 +444,7 @@ accounting_set_wakeup_time(void)
 
   if (! identity_key_is_set()) {
     if (init_keys() < 0) {
-      log_fn(LOG_ERR, "Error initializing keys");
+      log_err(LD_BUG, "Error initializing keys");
       tor_assert(0);
     }
   }
@@ -453,10 +466,11 @@ accounting_set_wakeup_time(void)
     time_to_exhaust_bw = 24*60*60;
     interval_wakeup_time = interval_start_time;
 
-    log_fn(LOG_NOTICE, "Configured hibernation.  This interval begins at %s "
-        "and ends at %s.  We have no prior estimate for bandwidth, so "
-        "we will start out awake and hibernate when we exhaust our quota.",
-        buf1, buf2);
+    log_notice(LD_ACCT,
+           "Configured hibernation.  This interval begins at %s "
+           "and ends at %s.  We have no prior estimate for bandwidth, so "
+           "we will start out awake and hibernate when we exhaust our quota.",
+           buf1, buf2);
     return;
   }
 
@@ -495,7 +509,8 @@ accounting_set_wakeup_time(void)
                    down_time<interval_end_time?down_time:interval_end_time);
     format_local_iso_time(buf4, interval_end_time);
 
-    log_fn(LOG_NOTICE, "Configured hibernation.  This interval began at %s; "
+    log_notice(LD_ACCT,
+           "Configured hibernation.  This interval began at %s; "
            "the scheduled wake-up time %s %s; "
            "we expect%s to exhaust our quota for this interval around %s; "
            "the next interval begins at %s (all times local)",
@@ -561,45 +576,45 @@ read_bandwidth_usage(void)
 
   if (smartlist_len(elts)<1 ||
       atoi(smartlist_get(elts,0)) != BW_ACCOUNTING_VERSION) {
-    log_fn(LOG_WARN, "Unrecognized bw_accounting file version: %s",
-           (const char*)smartlist_get(elts,0));
+    log_warn(LD_ACCT, "Unrecognized bw_accounting file version: %s",
+             (const char*)smartlist_get(elts,0));
     goto err;
   }
   if (smartlist_len(elts) < 7) {
-    log_fn(LOG_WARN, "Corrupted bw_accounting file: %d lines",
-           smartlist_len(elts));
+    log_warn(LD_ACCT, "Corrupted bw_accounting file: %d lines",
+             smartlist_len(elts));
     goto err;
   }
   if (parse_iso_time(smartlist_get(elts,1), &t1)) {
-    log_fn(LOG_WARN, "Error parsing bandwidth usage start time.");
+    log_warn(LD_ACCT, "Error parsing bandwidth usage start time.");
     goto err;
   }
   if (parse_iso_time(smartlist_get(elts,2), &t2)) {
-    log_fn(LOG_WARN, "Error parsing bandwidth usage last-written time");
+    log_warn(LD_ACCT, "Error parsing bandwidth usage last-written time");
     goto err;
   }
   n_read = tor_parse_uint64(smartlist_get(elts,3), 10, 0, UINT64_MAX,
                             &ok, NULL);
   if (!ok) {
-    log_fn(LOG_WARN, "Error parsing number of bytes read");
+    log_warn(LD_ACCT, "Error parsing number of bytes read");
     goto err;
   }
   n_written = tor_parse_uint64(smartlist_get(elts,4), 10, 0, UINT64_MAX,
                                &ok, NULL);
   if (!ok) {
-    log_fn(LOG_WARN, "Error parsing number of bytes read");
+    log_warn(LD_ACCT, "Error parsing number of bytes read");
     goto err;
   }
   n_seconds = (uint32_t)tor_parse_ulong(smartlist_get(elts,5), 10,0,ULONG_MAX,
                                         &ok, NULL);
   if (!ok) {
-    log_fn(LOG_WARN, "Error parsing number of seconds live");
+    log_warn(LD_ACCT, "Error parsing number of seconds live");
     goto err;
   }
   expected_bw =(uint32_t)tor_parse_ulong(smartlist_get(elts,6), 10,0,ULONG_MAX,
                                         &ok, NULL);
   if (!ok) {
-    log_fn(LOG_WARN, "Error parsing expected bandwidth");
+    log_warn(LD_ACCT, "Error parsing expected bandwidth");
     goto err;
   }
 
@@ -609,13 +624,18 @@ read_bandwidth_usage(void)
   interval_start_time = t1;
   expected_bandwidth_usage = expected_bw;
 
-  log_fn(LOG_INFO, "Successfully read bandwidth accounting file written at %s for interval starting at %s.  We have been active for %lu seconds in this interval.  At the start of the interval, we expected to use about %lu KB per second. ("U64_FORMAT" bytes read so far, "U64_FORMAT" bytes written so far)",
-         (char*)smartlist_get(elts,2),
-         (char*)smartlist_get(elts,1),
-         (unsigned long)n_seconds_active_in_interval,
-         (unsigned long)((uint64_t)expected_bandwidth_usage*1024/60),
-         U64_PRINTF_ARG(n_bytes_read_in_interval),
-         U64_PRINTF_ARG(n_bytes_written_in_interval));
+  log_info(LD_ACCT,
+       "Successfully read bandwidth accounting file written at %s "
+       "for interval starting at %s.  We have been active for %lu seconds in "
+       "this interval.  At the start of the interval, we expected to use "
+       "about %lu KB per second. ("U64_FORMAT" bytes read so far, "
+       U64_FORMAT" bytes written so far)",
+       (char*)smartlist_get(elts,2),
+       (char*)smartlist_get(elts,1),
+       (unsigned long)n_seconds_active_in_interval,
+       (unsigned long)((uint64_t)expected_bandwidth_usage*1024/60),
+       U64_PRINTF_ARG(n_bytes_read_in_interval),
+       U64_PRINTF_ARG(n_bytes_written_in_interval));
   SMARTLIST_FOREACH(elts, char *, cp, tor_free(cp));
   smartlist_free(elts);
 
@@ -640,7 +660,8 @@ hibernate_hard_limit_reached(void)
 
 /** Return true iff we have sent/received almost all the bytes we are willing
  * to send/receive this interval. */
-static int hibernate_soft_limit_reached(void)
+static int
+hibernate_soft_limit_reached(void)
 {
   uint64_t soft_limit = (uint64_t) ((get_options()->AccountingMax) * .95);
   if (!soft_limit)
@@ -652,15 +673,17 @@ static int hibernate_soft_limit_reached(void)
 /** Called when we get a SIGINT, or when bandwidth soft limit is
  * reached. Puts us into "loose hibernation": we don't accept new
  * connections, but we continue handling old ones. */
-static void hibernate_begin(int new_state, time_t now) {
+static void
+hibernate_begin(int new_state, time_t now)
+{
   connection_t *conn;
   or_options_t *options = get_options();
 
   if (new_state == HIBERNATE_STATE_EXITING &&
       hibernate_state != HIBERNATE_STATE_LIVE) {
-    log(LOG_NOTICE,"Sigint received %s; exiting now.",
-        hibernate_state == HIBERNATE_STATE_EXITING ?
-          "a second time" : "while hibernating");
+    log_notice(LD_GENERAL,"Sigint received %s; exiting now.",
+               hibernate_state == HIBERNATE_STATE_EXITING ?
+               "a second time" : "while hibernating");
     tor_cleanup();
     exit(0);
   }
@@ -669,7 +692,7 @@ static void hibernate_begin(int new_state, time_t now) {
   while ((conn = connection_get_by_type(CONN_TYPE_OR_LISTENER)) ||
          (conn = connection_get_by_type(CONN_TYPE_AP_LISTENER)) ||
          (conn = connection_get_by_type(CONN_TYPE_DIR_LISTENER))) {
-    log_fn(LOG_INFO,"Closing listener type %d", conn->type);
+    log_info(LD_NET,"Closing listener type %d", conn->type);
     connection_mark_for_close(conn);
   }
 
@@ -677,7 +700,8 @@ static void hibernate_begin(int new_state, time_t now) {
   /* XXX upload rendezvous service descriptors with no intro points */
 
   if (new_state == HIBERNATE_STATE_EXITING) {
-    log(LOG_NOTICE,"Interrupt: will shut down in %d seconds. Interrupt again to exit now.", options->ShutdownWaitLength);
+    log_notice(LD_GENERAL,"Interrupt: will shut down in %d seconds. Interrupt "
+               "again to exit now.", options->ShutdownWaitLength);
     hibernate_end_time = time(NULL) + options->ShutdownWaitLength;
   } else { /* soft limit reached */
     hibernate_end_time = interval_end_time;
@@ -689,13 +713,13 @@ static void hibernate_begin(int new_state, time_t now) {
 
 /** Called when we've been hibernating and our timeout is reached. */
 static void
-hibernate_end(int new_state) {
-
+hibernate_end(int new_state)
+{
   tor_assert(hibernate_state == HIBERNATE_STATE_LOWBANDWIDTH ||
              hibernate_state == HIBERNATE_STATE_DORMANT);
 
   /* listeners will be relaunched in run_scheduled_events() in main.c */
-  log_fn(LOG_NOTICE,"Hibernation period ended. Resuming normal activity.");
+  log_notice(LD_ACCT,"Hibernation period ended. Resuming normal activity.");
 
   hibernate_state = new_state;
   hibernate_end_time = 0; /* no longer hibernating */
@@ -704,20 +728,23 @@ hibernate_end(int new_state) {
 
 /** A wrapper around hibernate_begin, for when we get SIGINT. */
 void
-hibernate_begin_shutdown(void) {
+hibernate_begin_shutdown(void)
+{
   hibernate_begin(HIBERNATE_STATE_EXITING, time(NULL));
 }
 
 /** Return true iff we are currently hibernating. */
 int
-we_are_hibernating(void) {
+we_are_hibernating(void)
+{
   return hibernate_state != HIBERNATE_STATE_LIVE;
 }
 
 /** If we aren't currently dormant, close all connections and become
  * dormant. */
 static void
-hibernate_go_dormant(time_t now) {
+hibernate_go_dormant(time_t now)
+{
   connection_t *conn;
 
   if (hibernate_state == HIBERNATE_STATE_DORMANT)
@@ -727,7 +754,7 @@ hibernate_go_dormant(time_t now) {
   else
     hibernate_begin(HIBERNATE_STATE_DORMANT, now);
 
-  log_fn(LOG_NOTICE,"Going dormant. Blowing away remaining connections.");
+  log_notice(LD_ACCT,"Going dormant. Blowing away remaining connections.");
 
   /* Close all OR/AP/exit conns. Leave dir conns because we still want
    * to be able to upload server descriptors so people know we're still
@@ -740,7 +767,7 @@ hibernate_go_dormant(time_t now) {
     if (CONN_IS_EDGE(conn))
       connection_edge_end(conn, END_STREAM_REASON_HIBERNATING,
                           conn->cpath_layer);
-    log_fn(LOG_INFO,"Closing conn type %d", conn->type);
+    log_info(LD_NET,"Closing conn type %d", conn->type);
     if (conn->type == CONN_TYPE_AP) /* send socks failure if needed */
       connection_mark_unattached_ap(conn, END_STREAM_REASON_HIBERNATING);
     else
@@ -769,10 +796,14 @@ hibernate_end_time_elapsed(time_t now)
     format_iso_time(buf,interval_wakeup_time);
     if (hibernate_state != HIBERNATE_STATE_DORMANT) {
       /* We weren't sleeping before; we should sleep now. */
-      log_fn(LOG_NOTICE, "Accounting period ended. Commencing hibernation until %s GMT",buf);
+      log_notice(LD_ACCT,
+                 "Accounting period ended. Commencing hibernation until "
+                 "%s GMT", buf);
       hibernate_go_dormant(now);
     } else {
-      log_fn(LOG_NOTICE, "Accounting period ended. This period, we will hibernate until %s GMT",buf);
+      log_notice(LD_ACCT,
+             "Accounting period ended. This period, we will hibernate"
+             " until %s GMT",buf);
     }
   }
 }
@@ -780,7 +811,9 @@ hibernate_end_time_elapsed(time_t now)
 /** Consider our environment and decide if it's time
  * to start/stop hibernating.
  */
-void consider_hibernation(time_t now) {
+void
+consider_hibernation(time_t now)
+{
   int accounting_enabled = get_options()->AccountingMax != 0;
   char buf[ISO_TIME_LEN+1];
 
@@ -789,7 +822,7 @@ void consider_hibernation(time_t now) {
   if (hibernate_state == HIBERNATE_STATE_EXITING) {
     tor_assert(hibernate_end_time);
     if (hibernate_end_time <= now) {
-      log(LOG_NOTICE,"Clean shutdown finished. Exiting.");
+      log_notice(LD_GENERAL, "Clean shutdown finished. Exiting.");
       tor_cleanup();
       exit(0);
     }
@@ -812,11 +845,13 @@ void consider_hibernation(time_t now) {
    * go dormant. */
   if (hibernate_state == HIBERNATE_STATE_LIVE) {
     if (hibernate_soft_limit_reached()) {
-      log_fn(LOG_NOTICE,"Bandwidth soft limit reached; commencing hibernation.");
+      log_notice(LD_ACCT,
+                 "Bandwidth soft limit reached; commencing hibernation.");
       hibernate_begin(HIBERNATE_STATE_LOWBANDWIDTH, now);
     } else if (accounting_enabled && now < interval_wakeup_time) {
       format_iso_time(buf,interval_wakeup_time);
-      log_fn(LOG_NOTICE, "Commencing hibernation. We will wake up at %s GMT",buf);
+      log_notice(LD_ACCT,
+                 "Commencing hibernation. We will wake up at %s GMT", buf);
       hibernate_go_dormant(now);
     }
   }
@@ -833,3 +868,42 @@ void consider_hibernation(time_t now) {
   }
 }
 
+/** DOCDOC */
+int
+accounting_getinfo_helper(const char *question, char **answer)
+{
+  if (!strcmp(question, "accounting/enabled")) {
+    *answer = tor_strdup(get_options()->AccountingMax ? "1" : "0");
+  } else if (!strcmp(question, "accounting/hibernating")) {
+    if (hibernate_state == HIBERNATE_STATE_DORMANT)
+      *answer = tor_strdup("hard");
+    else if (hibernate_state == HIBERNATE_STATE_LOWBANDWIDTH)
+      *answer = tor_strdup("soft");
+    else
+      *answer = tor_strdup("awake");
+  } else if (!strcmp(question, "accounting/bytes")) {
+    *answer = tor_malloc(32);
+    tor_snprintf(*answer, 32, U64_FORMAT" "U64_FORMAT,
+                 U64_PRINTF_ARG(n_bytes_read_in_interval),
+                 U64_PRINTF_ARG(n_bytes_written_in_interval));
+  } else if (!strcmp(question, "accounting/bytes-left")) {
+    uint64_t limit = get_options()->AccountingMax;
+    *answer = tor_malloc(32);
+    tor_snprintf(*answer, 32, U64_FORMAT" "U64_FORMAT,
+                 U64_PRINTF_ARG(limit - n_bytes_read_in_interval),
+                 U64_PRINTF_ARG(limit - n_bytes_written_in_interval));
+  } else if (!strcmp(question, "accounting/interval-start")) {
+    *answer = tor_malloc(ISO_TIME_LEN+1);
+    format_iso_time(*answer, interval_start_time);
+  } else if (!strcmp(question, "accounting/interval-wake")) {
+    *answer = tor_malloc(ISO_TIME_LEN+1);
+    format_iso_time(*answer, interval_wakeup_time);
+  } else if (!strcmp(question, "accounting/interval-end")) {
+    *answer = tor_malloc(ISO_TIME_LEN+1);
+    format_iso_time(*answer, interval_end_time);
+  } else {
+    *answer = NULL;
+  }
+  return 0;
+}
+
