@@ -1,7 +1,8 @@
-/* Copyright 2004 Roger Dingledine, Nick Mathewson. */
+/* Copyright 2004-2006 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: rendcommon.c,v 1.47 2005/05/03 10:04:08 arma Exp $ */
-const char rendcommon_c_id[] = "$Id: rendcommon.c,v 1.47 2005/05/03 10:04:08 arma Exp $";
+/* $Id: rendcommon.c,v 1.66 2006/02/21 06:23:57 arma Exp $ */
+const char rendcommon_c_id[] =
+  "$Id: rendcommon.c,v 1.66 2006/02/21 06:23:57 arma Exp $";
 
 /**
  * \file rendcommon.c
@@ -12,13 +13,16 @@ const char rendcommon_c_id[] = "$Id: rendcommon.c,v 1.47 2005/05/03 10:04:08 arm
 #include "or.h"
 
 /** Return 0 if one and two are the same service ids, else -1 or 1 */
-int rend_cmp_service_ids(const char *one, const char *two) {
+int
+rend_cmp_service_ids(const char *one, const char *two)
+{
   return strcasecmp(one,two);
 }
 
 /** Free the storage held by the service descriptor <b>desc</b>.
  */
-void rend_service_descriptor_free(rend_service_descriptor_t *desc)
+void
+rend_service_descriptor_free(rend_service_descriptor_t *desc)
 {
   int i;
   if (desc->pk)
@@ -29,6 +33,13 @@ void rend_service_descriptor_free(rend_service_descriptor_t *desc)
     }
     tor_free(desc->intro_points);
   }
+  if (desc->intro_point_extend_info) {
+    for (i=0; i < desc->n_intro_points; ++i) {
+      if (desc->intro_point_extend_info[i])
+        extend_info_free(desc->intro_point_extend_info[i]);
+    }
+    tor_free(desc->intro_point_extend_info);
+  }
   tor_free(desc);
 }
 
@@ -38,38 +49,53 @@ void rend_service_descriptor_free(rend_service_descriptor_t *desc)
  */
 int
 rend_encode_service_descriptor(rend_service_descriptor_t *desc,
+                               int version,
                                crypto_pk_env_t *key,
                                char **str_out, size_t *len_out)
 {
-  char *buf, *cp, *ipoint;
+  char *cp;
+  char *end;
   int i;
-  size_t keylen, asn1len;
-  keylen = crypto_pk_keysize(desc->pk);
-  buf = tor_malloc(keylen*2); /* Too long, but that's okay. */
-  i = crypto_pk_asn1_encode(desc->pk, buf, keylen*2);
-  if (i<0) {
-    tor_free(buf);
-    return -1;
+  size_t asn1len;
+  size_t buflen = PK_BYTES*2*(desc->n_intro_points+2);/*Too long, but ok*/
+  cp = *str_out = tor_malloc(buflen);
+  end = cp + PK_BYTES*2*(desc->n_intro_points+1);
+  if (version) {
+    *(uint8_t*)cp = (uint8_t)0xff;
+    *(uint8_t*)(cp+1) = (uint8_t)version;
+    cp += 2;
   }
-  asn1len = i;
-  *len_out = 2 + asn1len + 4 + 2 + keylen;
-  for (i = 0; i < desc->n_intro_points; ++i) {
-    *len_out += strlen(desc->intro_points[i]) + 1;
-  }
-  cp = *str_out = tor_malloc(*len_out);
+  asn1len = crypto_pk_asn1_encode(desc->pk, cp+2, end-(cp+2));
   set_uint16(cp, htons((uint16_t)asn1len));
-  cp += 2;
-  memcpy(cp, buf, asn1len);
-  tor_free(buf);
-  cp += asn1len;
+  cp += 2+asn1len;
   set_uint32(cp, htonl((uint32_t)desc->timestamp));
   cp += 4;
+  if (version == 1) {
+    set_uint16(cp, htons(desc->protocols));
+    cp += 2;
+  }
   set_uint16(cp, htons((uint16_t)desc->n_intro_points));
   cp += 2;
-  for (i=0; i < desc->n_intro_points; ++i) {
-    ipoint = (char*)desc->intro_points[i];
-    strlcpy(cp, ipoint, *len_out-(cp-*str_out));
-    cp += strlen(ipoint)+1;
+  if (version == 0) {
+    for (i=0; i < desc->n_intro_points; ++i) {
+      char *ipoint = (char*)desc->intro_points[i];
+      strlcpy(cp, ipoint, buflen-(cp-*str_out));
+      cp += strlen(ipoint)+1;
+    }
+  } else {
+    if (desc->n_intro_points)
+      tor_assert(desc->intro_point_extend_info);
+    for (i=0; i < desc->n_intro_points; ++i) {
+      extend_info_t *info = desc->intro_point_extend_info[i];
+      int klen;
+      set_uint32(cp, htonl(info->addr));
+      set_uint16(cp+4, htons(info->port));
+      memcpy(cp+6, info->identity_digest, DIGEST_LEN);
+      klen = crypto_pk_asn1_encode(info->onion_key, cp+6+DIGEST_LEN+2,
+                                   (end-(cp+6+DIGEST_LEN+2)));
+      set_uint16(cp+6+DIGEST_LEN, htons((uint16_t)klen));
+      cp += 6+DIGEST_LEN+2+klen;
+    }
   }
   i = crypto_pk_private_sign_digest(key, cp, *str_out, cp-*str_out);
   if (i<0) {
@@ -77,7 +103,7 @@ rend_encode_service_descriptor(rend_service_descriptor_t *desc,
     return -1;
   }
   cp += i;
-  tor_assert(*len_out == (size_t)(cp-*str_out));
+  *len_out = (size_t)(cp-*str_out);
   return 0;
 }
 
@@ -85,17 +111,25 @@ rend_encode_service_descriptor(rend_service_descriptor_t *desc,
  * success, return a newly alloced service_descriptor_t.  On failure,
  * return NULL.
  */
-rend_service_descriptor_t *rend_parse_service_descriptor(
-                           const char *str, size_t len)
+rend_service_descriptor_t *
+rend_parse_service_descriptor(const char *str, size_t len)
 {
   rend_service_descriptor_t *result = NULL;
   int i;
   size_t keylen, asn1len;
   const char *end, *cp, *eos;
+  int version = 0;
 
   result = tor_malloc_zero(sizeof(rend_service_descriptor_t));
   cp = str;
   end = str+len;
+  if (end-cp<2) goto truncated;
+  if (*(uint8_t*)cp == 0xff) {
+    result->version = version = *(uint8_t*)(cp+1);
+    cp += 2;
+  } else {
+    result->version = version = 0;
+  }
   if (end-cp < 2) goto truncated;
   asn1len = ntohs(get_uint16(cp));
   cp += 2;
@@ -106,35 +140,75 @@ rend_service_descriptor_t *rend_parse_service_descriptor(
   if (end-cp < 4) goto truncated;
   result->timestamp = (time_t) ntohl(get_uint32(cp));
   cp += 4;
+  if (version == 1) {
+    if (end-cp < 2) goto truncated;
+    result->protocols = ntohs(get_uint16(cp));
+    cp += 2;
+  } else {
+    result->protocols = 1;
+  }
   if (end-cp < 2) goto truncated;
   result->n_intro_points = ntohs(get_uint16(cp));
-  result->intro_points = tor_malloc_zero(sizeof(char*)*result->n_intro_points);
   cp += 2;
-  for (i=0;i<result->n_intro_points;++i) {
-    if (end-cp < 2) goto truncated;
-    eos = (const char *)memchr(cp,'\0',end-cp);
-    if (!eos) goto truncated;
-    result->intro_points[i] = tor_strdup(cp);
-    cp = eos+1;
+
+  if (version == 0 && result->n_intro_points != 0) {
+    result->intro_points =
+      tor_malloc_zero(sizeof(char*)*result->n_intro_points);
+    for (i=0;i<result->n_intro_points;++i) {
+      if (end-cp < 2) goto truncated;
+      eos = (const char *)memchr(cp,'\0',end-cp);
+      if (!eos) goto truncated;
+      result->intro_points[i] = tor_strdup(cp);
+      cp = eos+1;
+    }
+  } else if (version != 0 && result->n_intro_points != 0) {
+    result->intro_point_extend_info =
+      tor_malloc_zero(sizeof(extend_info_t*)*result->n_intro_points);
+    result->intro_points =
+      tor_malloc_zero(sizeof(char*)*result->n_intro_points);
+    for (i=0;i<result->n_intro_points;++i) {
+      extend_info_t *info = result->intro_point_extend_info[i] =
+        tor_malloc_zero(sizeof(extend_info_t));
+      int klen;
+      if (end-cp < 8+DIGEST_LEN) goto truncated;
+      info->addr = ntohl(get_uint32(cp));
+      info->port = ntohs(get_uint16(cp+4));
+      memcpy(info->identity_digest, cp+6, DIGEST_LEN);
+      info->nickname[0] = '$';
+      base16_encode(info->nickname+1, sizeof(info->nickname)-1,
+                    info->identity_digest, DIGEST_LEN);
+      result->intro_points[i] = tor_strdup(info->nickname);
+      klen = ntohs(get_uint16(cp+6+DIGEST_LEN));
+      cp += 8+DIGEST_LEN;
+      if (end-cp < klen) goto truncated;
+      if (!(info->onion_key = crypto_pk_asn1_decode(cp,klen))) {
+        log_warn(LD_PROTOCOL,
+                 "Internal error decoding onion key for intro point.");
+        goto error;
+      }
+      cp += klen;
+    }
   }
   keylen = crypto_pk_keysize(result->pk);
   tor_assert(end-cp >= 0);
   if ((size_t)(end-cp) < keylen) goto truncated;
   if ((size_t)(end-cp) > keylen) {
-    log_fn(LOG_WARN, "Signature too long on service descriptor");
+    log_warn(LD_PROTOCOL,
+             "Signature is %d bytes too long on service descriptor.",
+             (int)((size_t)(end-cp) - keylen));
     goto error;
   }
   if (crypto_pk_public_checksig_digest(result->pk,
                                        (char*)str,cp-str, /* data */
                                        (char*)cp,end-cp  /* signature*/
                                        )<0) {
-    log_fn(LOG_WARN, "Bad signature on service descriptor");
+    log_warn(LD_PROTOCOL, "Bad signature on service descriptor.");
     goto error;
   }
 
   return result;
  truncated:
-  log_fn(LOG_WARN, "Truncated service descriptor");
+  log_warn(LD_PROTOCOL, "Truncated service descriptor.");
  error:
   rend_service_descriptor_free(result);
   return NULL;
@@ -144,7 +218,8 @@ rend_service_descriptor_t *rend_parse_service_descriptor(
  * base32 encoded.  NUL-terminates out.  (We use this string to
  * identify services in directory requests and .onion URLs.)
  */
-int rend_get_service_id(crypto_pk_env_t *pk, char *out)
+int
+rend_get_service_id(crypto_pk_env_t *pk, char *out)
 {
   char buf[DIGEST_LEN];
   tor_assert(pk);
@@ -165,11 +240,13 @@ static strmap_t *rend_cache = NULL;
 
 /** Initializes the service descriptor cache.
  */
-void rend_cache_init(void)
+void
+rend_cache_init(void)
 {
   rend_cache = strmap_new();
 }
 
+/** Helper: free storage held by a single service descriptor cache entry. */
 static void
 _rend_cache_entry_free(void *p)
 {
@@ -179,6 +256,7 @@ _rend_cache_entry_free(void *p)
   tor_free(e);
 }
 
+/** Free all storage held by the service descriptor cache. */
 void
 rend_cache_free_all(void)
 {
@@ -188,7 +266,8 @@ rend_cache_free_all(void)
 
 /** Removes all old entries from the service descriptor cache.
  */
-void rend_cache_clean(void)
+void
+rend_cache_clean(void)
 {
   strmap_iter_t *iter;
   const char *key;
@@ -210,7 +289,9 @@ void rend_cache_clean(void)
 
 /** Return true iff <b>query</b> is a syntactically valid service ID (as
  * generated by rend_get_service_id).  */
-int rend_valid_service_id(const char *query) {
+int
+rend_valid_service_id(const char *query)
+{
   if (strlen(query) != REND_SERVICE_ID_LEN)
     return 0;
 
@@ -220,15 +301,28 @@ int rend_valid_service_id(const char *query) {
   return 1;
 }
 
-/** If we have a cached rend_cache_entry_t for the service ID <b>query</b>, set
- * *<b>e</b> to that entry and return 1.  Else return 0.
+/** If we have a cached rend_cache_entry_t for the service ID <b>query</b>,
+ * set *<b>e</b> to that entry and return 1.  Else return 0.  If
+ * <b>version</b> is nonnegative, only return an entry in that descriptor
+ * format version. Otherwise (if <b>version</b> is negative), return the most
+ * recent format we have.
  */
-int rend_cache_lookup_entry(const char *query, rend_cache_entry_t **e)
+int
+rend_cache_lookup_entry(const char *query, int version, rend_cache_entry_t **e)
 {
+  char key[REND_SERVICE_ID_LEN+2];
   tor_assert(rend_cache);
   if (!rend_valid_service_id(query))
     return -1;
-  *e = strmap_get_lc(rend_cache, query);
+  *e = NULL;
+  if (version != 0) {
+    tor_snprintf(key, sizeof(key), "1%s", query);
+    *e = strmap_get_lc(rend_cache, key);
+  }
+  if (!*e && version != 1) {
+    tor_snprintf(key, sizeof(key), "0%s", query);
+    *e = strmap_get_lc(rend_cache, key);
+  }
   if (!*e)
     return 0;
   return 1;
@@ -242,11 +336,13 @@ int rend_cache_lookup_entry(const char *query, rend_cache_entry_t **e)
  * Note: calls to rend_cache_clean or rend_cache_store may invalidate
  * *desc.
  */
-int rend_cache_lookup_desc(const char *query, const char **desc, size_t *desc_len)
+int
+rend_cache_lookup_desc(const char *query, int version, const char **desc,
+                       size_t *desc_len)
 {
   rend_cache_entry_t *e;
   int r;
-  r = rend_cache_lookup_entry(query,&e);
+  r = rend_cache_lookup_entry(query,version,&e);
   if (r <= 0) return r;
   *desc = e->desc;
   *desc_len = e->len;
@@ -260,51 +356,57 @@ int rend_cache_lookup_desc(const char *query, const char **desc, size_t *desc_le
  * it's the same or older than one we've already got; return 1 if
  * it's novel.
  */
-int rend_cache_store(const char *desc, size_t desc_len)
+int
+rend_cache_store(const char *desc, size_t desc_len)
 {
   rend_cache_entry_t *e;
   rend_service_descriptor_t *parsed;
   char query[REND_SERVICE_ID_LEN+1];
+  char key[REND_SERVICE_ID_LEN+2];
   time_t now;
 
   tor_assert(rend_cache);
   parsed = rend_parse_service_descriptor(desc,desc_len);
   if (!parsed) {
-    log_fn(LOG_WARN,"Couldn't parse service descriptor");
+    log_warn(LD_PROTOCOL,"Couldn't parse service descriptor.");
     return -1;
   }
   if (rend_get_service_id(parsed->pk, query)<0) {
-    log_fn(LOG_WARN,"Couldn't compute service ID");
+    log_warn(LD_BUG,"Couldn't compute service ID.");
     rend_service_descriptor_free(parsed);
     return -1;
   }
+  tor_snprintf(key, sizeof(key), "%c%s", parsed->version?'1':'0', query);
   now = time(NULL);
   if (parsed->timestamp < now-REND_CACHE_MAX_AGE-REND_CACHE_MAX_SKEW) {
-    log_fn(LOG_WARN,"Service descriptor %s is too old", safe_str(query));
+    log_fn(LOG_PROTOCOL_WARN, LD_REND,
+           "Service descriptor %s is too old.", safe_str(query));
     rend_service_descriptor_free(parsed);
     return -1;
   }
   if (parsed->timestamp > now+REND_CACHE_MAX_SKEW) {
-    log_fn(LOG_WARN,"Service descriptor %s is too far in the future",
-           safe_str(query));
+    log_fn(LOG_PROTOCOL_WARN, LD_REND,
+           "Service descriptor %s is too far in the future.", safe_str(query));
     rend_service_descriptor_free(parsed);
     return -1;
   }
-  e = (rend_cache_entry_t*) strmap_get_lc(rend_cache, query);
+  e = (rend_cache_entry_t*) strmap_get_lc(rend_cache, key);
   if (e && e->parsed->timestamp > parsed->timestamp) {
-    log_fn(LOG_INFO,"We already have a newer service descriptor %s with the same ID", safe_str(query));
+    log_info(LD_REND,"We already have a newer service descriptor %s with the "
+             "same ID and version.", safe_str(query));
     rend_service_descriptor_free(parsed);
     return 0;
   }
   if (e && e->len == desc_len && !memcmp(desc,e->desc,desc_len)) {
-    log_fn(LOG_INFO,"We already have this service descriptor %s", safe_str(query));
+    log_info(LD_REND,"We already have this service descriptor %s.",
+             safe_str(query));
     e->received = time(NULL);
     rend_service_descriptor_free(parsed);
     return 0;
   }
   if (!e) {
     e = tor_malloc_zero(sizeof(rend_cache_entry_t));
-    strmap_set_lc(rend_cache, query, e);
+    strmap_set_lc(rend_cache, key, e);
   } else {
     rend_service_descriptor_free(e->parsed);
     tor_free(e->desc);
@@ -315,15 +417,16 @@ int rend_cache_store(const char *desc, size_t desc_len)
   e->desc = tor_malloc(desc_len);
   memcpy(e->desc, desc, desc_len);
 
-  log_fn(LOG_INFO,"Successfully stored rend desc '%s', len %d",
-         safe_str(query), (int)desc_len);
+  log_debug(LD_REND,"Successfully stored rend desc '%s', len %d.",
+            safe_str(query), (int)desc_len);
   return 1;
 }
 
 /** Called when we get a rendezvous-related relay cell on circuit
  * <b>circ</b>.  Dispatch on rendezvous relay command. */
-void rend_process_relay_cell(circuit_t *circ, int command, size_t length,
-                             const char *payload)
+void
+rend_process_relay_cell(circuit_t *circ, int command, size_t length,
+                        const char *payload)
 {
   int r;
   switch (command) {
