@@ -1,6 +1,6 @@
 /* Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: tor-resolve.c,v 1.17 2005/04/01 20:15:56 nickm Exp $ */
+/* $Id: tor-resolve.c,v 1.24 2006/02/13 10:36:39 arma Exp $ */
 
 #include "orconfig.h"
 
@@ -40,10 +40,13 @@
 #endif
 
 #define RESPONSE_LEN 8
-#define log_sock_error(act, _s)                             \
-  do { log_fn(LOG_ERR, "Error while %s: %s", act,           \
+#define log_sock_error(act, _s)                                         \
+  do { log_fn(LOG_ERR, LD_NET, "Error while %s: %s", act,              \
               tor_socket_strerror(tor_socket_errno(_s))); } while (0)
 
+/** Set *<b>out</b> to a newly allocated SOCKS4a resolve request with
+ * <b>username</b> and <b>hostname</b> as provided.  Return the number
+ * of bytes in the request. */
 static int
 build_socks4a_resolve_request(char **out,
                               const char *username,
@@ -66,6 +69,10 @@ build_socks4a_resolve_request(char **out,
   return len;
 }
 
+/** Given a <b>len</b>-byte SOCKS4a response in <b>response</b>, set
+ * *<b>addr_out</b> to the address it contains (in host order).
+ * Return 0 on success, -1 on error.
+ */
 static int
 parse_socks4a_resolve_response(const char *response, size_t len,
                                uint32_t *addr_out)
@@ -75,20 +82,20 @@ parse_socks4a_resolve_response(const char *response, size_t len,
   tor_assert(addr_out);
 
   if (len < RESPONSE_LEN) {
-    log_fn(LOG_WARN,"Truncated socks response.");
+    log_warn(LD_PROTOCOL,"Truncated socks response.");
     return -1;
   }
   if (((uint8_t)response[0])!=0) { /* version: 0 */
-    log_fn(LOG_WARN,"Nonzero version in socks response: bad format.");
+    log_warn(LD_PROTOCOL,"Nonzero version in socks response: bad format.");
     return -1;
   }
   status = (uint8_t)response[1];
   if (get_uint16(response+2)!=0) { /* port: 0 */
-    log_fn(LOG_WARN,"Nonzero port in socks response: bad format.");
+    log_warn(LD_PROTOCOL,"Nonzero port in socks response: bad format.");
     return -1;
   }
   if (status != 90) {
-    log_fn(LOG_WARN,"Got status response '%d': socks request failed.", status);
+    log_warn(LD_NET,"Got status response '%d': socks request failed.", status);
     return -1;
   }
 
@@ -96,6 +103,10 @@ parse_socks4a_resolve_response(const char *response, size_t len,
   return 0;
 }
 
+/** Send a resolve request for <b>hostname</b> to the Tor listening on
+ * <b>sockshost</b>:<b>socksport</b>.  Store the resulting IPv4
+ * address (in host order) into *<b>result_addr</b>.
+ */
 static int
 do_resolve(const char *hostname, uint32_t sockshost, uint16_t socksport,
            uint32_t *result_addr)
@@ -125,7 +136,7 @@ do_resolve(const char *hostname, uint32_t sockshost, uint16_t socksport,
   }
 
   if ((len = build_socks4a_resolve_request(&req, "", hostname))<0) {
-    log_fn(LOG_ERR, "Error generating SOCKS request");
+    log_err(LD_BUG,"Error generating SOCKS request");
     return -1;
   }
 
@@ -146,7 +157,7 @@ do_resolve(const char *hostname, uint32_t sockshost, uint16_t socksport,
   while (len < RESPONSE_LEN) {
     r = recv(s, response_buf+len, RESPONSE_LEN-len, 0);
     if (r==0) {
-      log_fn(LOG_WARN,"EOF while reading SOCKS response");
+      log_warn(LD_NET,"EOF while reading SOCKS response");
       return -1;
     }
     if (r<0) {
@@ -156,13 +167,15 @@ do_resolve(const char *hostname, uint32_t sockshost, uint16_t socksport,
     len += r;
   }
 
-  if (parse_socks4a_resolve_response(response_buf, RESPONSE_LEN,result_addr)<0){
+  if (parse_socks4a_resolve_response(response_buf, RESPONSE_LEN,
+                                     result_addr)<0){
     return -1;
   }
 
   return 0;
 }
 
+/** Print a usage message and exit. */
 static void
 usage(void)
 {
@@ -170,6 +183,7 @@ usage(void)
   exit(1);
 }
 
+/** Entry point to tor-resolve */
 int
 main(int argc, char **argv)
 {
@@ -199,7 +213,7 @@ main(int argc, char **argv)
     add_stream_log(LOG_WARN, LOG_ERR, "<stderr>", stderr);
   }
   if (n_args == 1) {
-    log_fn(LOG_DEBUG, "defaulting to localhost:9050");
+    log_debug(LD_CONFIG, "defaulting to localhost:9050");
     sockshost = 0x7f000001u; /* localhost */
     socksport = 9050; /* 9050 */
   } else if (n_args == 2) {
@@ -208,7 +222,7 @@ main(int argc, char **argv)
       return 1;
     }
     if (socksport == 0) {
-      log_fn(LOG_DEBUG, "defaulting to port 9050");
+      log_debug(LD_CONFIG, "defaulting to port 9050");
       socksport = 9050;
     }
   } else {
@@ -216,14 +230,15 @@ main(int argc, char **argv)
   }
 
   if (!strcasecmpend(arg[0], ".onion")) {
-    fprintf(stderr, "%s is a hidden service; those don't have IP addresses.\n\n"
-       "To connect to a hidden service, you need to send the hostname to Tor;\n"
-       "we suggest SOCKS 4a.\n", arg[0]);
+    fprintf(stderr,
+       "%s is a hidden service; those don't have IP addresses.\n\n"
+       "To connect to a hidden service, you need to send the hostname\n"
+       "to Tor; we suggest an application that uses SOCKS 4a.\n", arg[0]);
     return 1;
   }
 
   if (network_init()<0) {
-    log_fn(LOG_ERR,"Error initializing network; exiting.");
+    log_err(LD_NET,"Error initializing network; exiting.");
     return 1;
   }
 
@@ -235,3 +250,4 @@ main(int argc, char **argv)
   printf("%s\n", buf);
   return 0;
 }
+
